@@ -54,7 +54,7 @@ int     interactive, caseins, lex_compat, posix_compat, do_yylineno,
 int     fullspd, gen_line_dirs, performance_report, backing_up_report;
 int     C_plus_plus, long_align, use_read, yytext_is_array, do_yywrap,
 	csize;
-int     reentrant, bison_bridge;
+int     reentrant, bison_bridge_lval, bison_bridge_lloc;
 int     yymore_used, reject, real_reject, continued_action, in_rule;
 int     yymore_really_used, reject_really_used;
 int     datapos, dataline, linenum, out_linenum;
@@ -105,6 +105,7 @@ int     num_input_files;
 jmp_buf flex_main_jmp_buf;
 bool   *rule_has_nl, *ccl_has_nl;
 int     nlch = '\n';
+bool    ansi_func_defs, ansi_func_protos;
 
 bool    tablesext, tablesverify, gentables;
 char   *tablesfilename=0,*tablesname=0;
@@ -129,19 +130,25 @@ static char *tablesfile_template = "lex%s.tbl";
 extern unsigned _stklen = 16384;
 #endif
 
+/* From scan.l */
+extern FILE* yyout;
+
 static char outfile_path[MAXLINE];
 static int outfile_created = 0;
 static char *skelname = NULL;
+static int _stdout_closed = 0; /* flag to prevent double-fclose() on stdout. */
+
+/* For debugging. The max number of filters to apply to skeleton. */
+static int preproc_level = 1000;
 
 int flex_main PROTO ((int argc, char *argv[]));
 int main PROTO ((int argc, char *argv[]));
-void fix_line_dirs PROTO ((char *, char *, char *, int));
 
 int flex_main (argc, argv)
      int argc;
      char   *argv[];
 {
-	int     i, exit_status;
+	int     i, exit_status, child_status;
 
 	/* Set a longjmp target. Yes, I know it's a hack, but it gets worse: The
 	 * return value of setjmp, if non-zero, is the desired exit code PLUS ONE.
@@ -151,8 +158,23 @@ int flex_main (argc, argv)
 	 * exit(n);
 	 */
 	exit_status = setjmp (flex_main_jmp_buf);
-	if (exit_status)
-		return exit_status - 1;
+	if (exit_status){
+        if (stdout && !_stdout_closed && !ferror(stdout)){
+            fflush(stdout);
+            fclose(stdout);
+        }
+        while (wait(&child_status) > 0){
+            if (!WIFEXITED (child_status)
+                || WEXITSTATUS (child_status) != 0){
+                /* report an error of a child
+                 */
+                if( exit_status <= 1 )
+                    exit_status = 2;
+
+            }
+        }
+        return exit_status - 1;
+    }
 
 	flexinit (argc, argv);
 
@@ -187,9 +209,12 @@ int main (argc, argv)
      char   *argv[];
 {
 #if ENABLE_NLS
+#if HAVE_LOCALE_H
 	setlocale (LC_MESSAGES, "");
+        setlocale (LC_CTYPE, "");
 	textdomain (PACKAGE);
 	bindtextdomain (PACKAGE, LOCALEDIR);
+#endif
 #endif
 
 	return flex_main (argc, argv);
@@ -200,6 +225,7 @@ int main (argc, argv)
 void check_options ()
 {
 	int     i;
+    const char * m4 = NULL;
 
 	if (lex_compat) {
 		if (C_plus_plus)
@@ -208,7 +234,7 @@ void check_options ()
 		if (fulltbl || fullspd)
 			flexerror (_("Can't use -f or -F with -l option"));
 
-		if (reentrant || bison_bridge)
+		if (reentrant || bison_bridge_lval)
 			flexerror (_
 				   ("Can't use --reentrant or --bison-bridge with -l option"));
 
@@ -222,10 +248,6 @@ void check_options ()
 		use_read = false;
 	}
 
-	/* See comments in flexend() for an explanation of this error condition. */
-	if (use_stdout && headerfilename)
-		flexerror (_
-			   ("Can't specify header option if writing to stdout."));
 
 #if 0
 	/* This makes no sense whatsoever. I'm removing it. */
@@ -277,7 +299,7 @@ void check_options ()
 	if (C_plus_plus && (reentrant))
 		flexerror (_("Options -+ and --reentrant are mutually exclusive."));
 
-	if (C_plus_plus && bison_bridge)
+	if (C_plus_plus && bison_bridge_lval)
 		flexerror (_("bison bridge not supported for the C++ scanner."));
 
 
@@ -304,6 +326,12 @@ void check_options ()
 		}
 	}
 
+    if (!ansi_func_defs)
+        buf_m4_define( &m4defs_buf, "M4_YY_NO_ANSI_FUNC_DEFS", NULL);
+
+    if (!ansi_func_protos)
+        buf_m4_define( &m4defs_buf, "M4_YY_NO_ANSI_FUNC_PROTOS", NULL);
+
 	if (!use_stdout) {
 		FILE   *prev_stdout;
 
@@ -329,8 +357,24 @@ void check_options ()
 		outfile_created = 1;
 	}
 
+
+    /* Setup the filter chain. */
+    output_chain = filter_create_int(NULL, filter_tee_header, headerfilename);
+    if ( !(m4 = getenv("M4")))
+        m4 = M4;
+    filter_create_ext(output_chain, m4, "-P", 0);
+    filter_create_int(output_chain, filter_fix_linedirs, NULL);
+
+    /* For debugging, only run the requested number of filters. */
+    if (preproc_level > 0) {
+        filter_truncate(output_chain, preproc_level);
+        filter_apply_chain(output_chain);
+    }
+    yyout = stdout;
+
+
 	/* always generate the tablesverify flag. */
-	action_define ("YY_TABLES_VERIFY", tablesverify ? 1 : 0);
+	buf_m4_define (&m4defs_buf, "M4_YY_TABLES_VERIFY", tablesverify ? "1" : "0");
 	if (tablesext)
 		gentables = false;
 
@@ -345,7 +389,7 @@ void check_options ()
 		char   *pname = 0;
 		int     nbytes = 0;
 
-		action_define ("YY_TABLES_EXTERNAL", 1);
+		buf_m4_define (&m4defs_buf, "M4_YY_TABLES_EXTERNAL", NULL);
 
 		if (!tablesfilename) {
 			nbytes = strlen (prefix) +
@@ -376,93 +420,58 @@ void check_options ()
 		lerrsf (_("can't open skeleton file %s"), skelname);
 
 	if (reentrant) {
-		outn ("#define YY_REENTRANT 1");
+        buf_m4_define (&m4defs_buf, "M4_YY_REENTRANT", NULL);
 		if (yytext_is_array)
-			outn ("#define YY_TEXT_IS_ARRAY");
+			buf_m4_define (&m4defs_buf, "M4_YY_TEXT_IS_ARRAY", NULL);
 	}
 
-	if ( bison_bridge)
-		outn ("#define YY_BISON_BRIDGE 1");
+	if ( bison_bridge_lval)
+		buf_m4_define (&m4defs_buf, "M4_YY_BISON_LVAL", NULL);
 
-	if (strcmp (prefix, "yy")) {
-#define GEN_PREFIX(name) out_str3( "#define yy%s %s%s\n", name, prefix, name )
-		if (C_plus_plus)
-			GEN_PREFIX ("FlexLexer");
-		else {
-			outn ("#ifndef YY_REENTRANT");
-			GEN_PREFIX ("text");
-			GEN_PREFIX ("leng");
-			GEN_PREFIX ("in");
-			GEN_PREFIX ("out");
-			GEN_PREFIX ("_flex_debug");
-			outn ("#endif");
-			GEN_PREFIX ("_create_buffer");
-			GEN_PREFIX ("_delete_buffer");
-			GEN_PREFIX ("_scan_buffer");
-			GEN_PREFIX ("_scan_string");
-			GEN_PREFIX ("_scan_bytes");
-			GEN_PREFIX ("_init_buffer");
-			GEN_PREFIX ("_flush_buffer");
-			GEN_PREFIX ("_load_buffer_state");
-			GEN_PREFIX ("_switch_to_buffer");
-			GEN_PREFIX ("lex");
-			GEN_PREFIX ("restart");
-			GEN_PREFIX ("lex_init");
-			GEN_PREFIX ("lex_destroy");
-			GEN_PREFIX ("get_debug");
-			GEN_PREFIX ("set_debug");
-			GEN_PREFIX ("get_extra");
-			GEN_PREFIX ("set_extra");
-			GEN_PREFIX ("get_in");
-			GEN_PREFIX ("set_in");
-			GEN_PREFIX ("get_out");
-			GEN_PREFIX ("set_out");
-			GEN_PREFIX ("get_leng");
-			GEN_PREFIX ("get_text");
-			GEN_PREFIX ("get_lineno");
-			GEN_PREFIX ("set_lineno");
+	if ( bison_bridge_lloc)
+        buf_m4_define (&m4defs_buf, "<M4_YY_BISON_LLOC>", NULL);
 
-			outn ("#ifdef YY_BISON_BRIDGE");
-			GEN_PREFIX ("get_lval");
-			GEN_PREFIX ("set_lval");
-			GEN_PREFIX ("get_lloc");
-			GEN_PREFIX ("set_lloc");
-			outn ("#endif");
-
-		}
-
-        /* The alloc/realloc/free functions are used internally by the
-         * generated scanner for both and C++.
-         */
-        GEN_PREFIX ("alloc");
-        GEN_PREFIX ("realloc");
-        GEN_PREFIX ("free");
-
-		if (!reentrant)
-			GEN_PREFIX ("lineno");
-
-		if (do_yywrap)
-			GEN_PREFIX ("wrap");
-
-        if (tablesext){
-            GEN_PREFIX ("tables_fload");
-            GEN_PREFIX ("tables_destroy");
-            GEN_PREFIX ("TABLES_NAME");
-        }
-
-		outn ("");
-	}
+    buf_m4_define(&m4defs_buf, "M4_YY_PREFIX", prefix);
 
 	if (did_outfilename)
 		line_directive_out (stdout, 0);
 
 	if (do_yylineno)
-		buf_strdefine (&userdef_buf, "YY_USE_LINENO", "1");
+		buf_m4_define (&m4defs_buf, "M4_YY_USE_LINENO", NULL);
 
 	/* Create the alignment type. */
 	buf_strdefine (&userdef_buf, "YY_INT_ALIGNED",
 		       long_align ? "long int" : "short int");
 
+    /* Define the start condition macros. */
+    {
+        struct Buf tmpbuf;
+        buf_init(&tmpbuf, sizeof(char));
+        for (i = 1; i <= lastsc; i++) {
+             char *str, *fmt = "#define %s %d\n";
+
+             str = (char*)flex_alloc(strlen(fmt) + strlen(scname[i]) + (int)(1 + log10(i)) + 2);
+             sprintf(str, fmt,      scname[i], i - 1);
+             buf_strappend(&tmpbuf, str);
+             free(str);
+        }
+        buf_m4_define(&m4defs_buf, "M4_YY_SC_DEFS", tmpbuf.elts);
+        buf_destroy(&tmpbuf);
+    }
+
+    /* This is where we begin writing to the file. */
+
+    /* Dump the %top code. */
+    if( top_buf.elts)
+        outn((char*) top_buf.elts);
+
+    /* Dump the m4 definitions. */
+    buf_print_strings(&m4defs_buf, stdout);
+    m4defs_buf.nelts = 0; /* memory leak here. */
+
+    /* Place a bogus line directive, it will be fixed in the filter. */
+    outn("#line 0 \"M4_YY_OUTFILE_NAME\"\n");
+
 	/* Dump the user defined preproc directives. */
 	if (userdef_buf.elts)
 		outn ((char *) (userdef_buf.elts));
@@ -471,61 +480,6 @@ void check_options ()
 	/* %% [1.0] */
 }
 
-/* Alter #line directives from the generated source, destined
- * for the generated header. We chaneg the line number and filename.
- * linebuf is modified in place.
- */
-void fix_line_dirs (linebuf, outfilename, headerfilename, nlines)
-     char   *linebuf;
-     char   *outfilename;
-     char   *headerfilename;
-     int nlines;
-{
-	char   *pname, *p;
-
-	/* Match pattern:  ^#line +[:digit:]+ +"outfilename" */
-	p = linebuf;
-	if (strncmp (p, "#line ", 6))
-		return;
-	p += 6;
-
-	/* match spaces */
-	while (*p == ' ')
-		p++;
-	if (!isdigit (*p))
-		return;
-
-	/* match numbers */
-	while (isdigit (*p))
-		p++;
-	if (*p != ' ')
-		return;
-
-	/* match spaces */
-	while (*p == ' ')
-		p++;
-	if (*p != '"')
-		return;
-	p++;
-
-	pname = p;
-
-	/* find end of filename. Note: If scanner filename has
-	 * embedded '"' chars, then the generated #line directive
-	 * may fail. */
-	while (*p != '\0' && *p != '"')
-		p++;
-	if (*p != '"')
-		return;
-
-	if (strncmp (pname, outfilename, p - pname) != 0)
-		return;
-
-	/* We have a match. */
-
-	sprintf (linebuf, "#line %d \"%s\"\n", nlines + 2, headerfilename);
-}
-
 /* flexend - terminate flex
  *
  * note
@@ -538,7 +492,6 @@ void flexend (exit_status)
 {
 	static int called_before = -1;	/* prevent infinite recursion. */
 	int     tblsiz;
-	int     i;
 
 	if (++called_before)
 		FLEX_EXIT (exit_status);
@@ -553,61 +506,7 @@ void flexend (exit_status)
 				skelname);
 	}
 
-	/* flex generates the header file by rewinding the output FILE
-	 * pointer. However, since we can't rewind stdout, we must disallow
-	 * %option header if we are writing to stdout. This is a kludge.
-	 * This kludge can be rewritten when we get around to buffering
-	 * Section 1 of the input file, because then we'll have seen all the
-	 * %options BEFORE we begin generating the scanner. The lack of
-	 * buffering causes other problems, too. For example, it is the
-	 * reason we currently can't provide a mechanism to allow the user
-	 * to inject arbitrary class members into the generated C++ scanner. - JM
-	 */
-	if (headerfilename && exit_status == 0 && outfile_created
-	    && !ferror (stdout)) {
-		/* Copy the file we just wrote to a header file. */
-#define LINE_SZ 512
-		FILE   *header_out;
-		char    linebuf[LINE_SZ];
-		int     nlines = 0;
-		int     discard = 0;
-
-		/* rewind the outfile file. */
-		fflush (stdout);
-		fseek (stdout, 0L, SEEK_SET);
-
-		header_out = fopen (headerfilename, "w");
-		if (header_out == NULL)
-			lerrsf (_("could not create %s"), headerfilename);
-
-		fprintf (header_out, "#ifndef %sHEADER_H\n", prefix);
-		fprintf (header_out, "#define %sHEADER_H 1\n", prefix);
-		fprintf (header_out, "#define %sIN_HEADER 1\n\n", prefix);
-		fflush (header_out);
-
-		nlines = 4;
-		while (fgets (linebuf, LINE_SZ, stdout)) {
-			if (strstr (linebuf, "YY-DISCARD-FROM-HEADER"))
-				discard++;
-			else if (strstr
-				 (linebuf, "YY-END-DISCARD-FROM-HEADER")) {
-				discard--;
-				continue;
-			}
-
-			if (discard)
-				continue;
-
-			fix_line_dirs (linebuf, outfilename,
-				       headerfilename, nlines);
-			fputs (linebuf, header_out);
-			nlines++;
-		}
-
-		fprintf (header_out, "#line %d \"%s\"\n", (++nlines) + 1,
-			 headerfilename);
-
-		/* Print the start conditions. */
+#if 0
 		fprintf (header_out,
 			 "#ifdef YY_HEADER_EXPORT_START_CONDITIONS\n");
 		fprintf (header_out,
@@ -617,8 +516,7 @@ void flexend (exit_status)
 		fprintf (header_out,
 			 "#undef INITIAL\n#define INITIAL 0\n");
 		for (i = 2; i <= lastsc; i++)
-			fprintf (header_out, "#define %s %d\n", scname[i],
-				 i - 1);
+			fprintf (header_out, "#define %s %d\n", scname[i], i - 1);
 		fprintf (header_out,
 			 "#endif /* YY_HEADER_EXPORT_START_CONDITIONS */\n\n");
 
@@ -628,151 +526,148 @@ void flexend (exit_status)
 		fprintf (header_out,
 			 "/* Undefine all internal macros, etc., that do no belong in the header. */\n\n");
 
-		fprintf (header_out, "#undef BEGIN\n");
-		fprintf (header_out, "#undef ECHO\n");
-		fprintf (header_out, "#undef EOB_ACT_CONTINUE_SCAN\n");
-		fprintf (header_out, "#undef EOB_ACT_END_OF_FILE\n");
-		fprintf (header_out, "#undef EOB_ACT_LAST_MATCH\n");
-		fprintf (header_out, "#undef FLEX_SCANNER\n");
-		fprintf (header_out, "#undef FLEX_STD\n");
-		fprintf (header_out, "#undef REJECT\n");
-		fprintf (header_out, "#undef YYLMAX\n");
-		fprintf (header_out, "#undef YYSTATE\n");
-		fprintf (header_out, "#undef YY_AT_BOL\n");
-		fprintf (header_out, "#undef YY_BREAK\n");
-		fprintf (header_out, "#undef YY_BUFFER_EOF_PENDING\n");
-		fprintf (header_out, "#undef YY_BUFFER_NEW\n");
-		fprintf (header_out, "#undef YY_BUFFER_NORMAL\n");
-		fprintf (header_out, "#undef YY_BUF_SIZE\n");
-		fprintf (header_out, "#undef YY_CALL_LAST_ARG\n");
-		fprintf (header_out, "#undef YY_CALL_ONLY_ARG\n");
-		fprintf (header_out, "#undef YY_CURRENT_BUFFER\n");
-		fprintf (header_out, "#undef YY_DECL\n");
-		fprintf (header_out, "#undef YY_DECL_LAST_ARG\n");
-		fprintf (header_out, "#undef YY_DO_BEFORE_ACTION\n");
-		fprintf (header_out, "#undef YY_END_OF_BUFFER\n");
-		fprintf (header_out, "#undef YY_END_OF_BUFFER_CHAR\n");
-		fprintf (header_out, "#undef YY_EXIT_FAILURE\n");
-		fprintf (header_out, "#undef YY_EXTRA_TYPE\n");
-		fprintf (header_out, "#undef YY_FATAL_ERROR\n");
-		fprintf (header_out, "#undef YY_FLEX_DEFINED_ECHO\n");
-		fprintf (header_out, "#undef YY_FLEX_LEX_COMPAT\n");
-		fprintf (header_out, "#undef YY_FLEX_MAJOR_VERSION\n");
-		fprintf (header_out, "#undef YY_FLEX_MINOR_VERSION\n");
-		fprintf (header_out, "#undef YY_FLUSH_BUFFER\n");
-		fprintf (header_out, "#undef YY_G\n");
-		fprintf (header_out, "#undef YY_INPUT\n");
-		fprintf (header_out, "#undef YY_INT_ALIGNED\n");
-		fprintf (header_out, "#undef YY_INTERACTIVE\n");
-		fprintf (header_out, "#undef YY_LAST_ARG\n");
-		fprintf (header_out, "#undef YY_LEX_ARGS\n");
-		fprintf (header_out, "#undef YY_MAIN\n");
-		fprintf (header_out, "#undef YY_MORE_ADJ\n");
-		fprintf (header_out, "#undef YY_NEED_STRLEN\n");
-		fprintf (header_out, "#undef YY_NEW_FILE\n");
-		fprintf (header_out, "#undef YY_NO_FLEX_ALLOC\n");
-		fprintf (header_out, "#undef YY_NO_FLEX_REALLOC\n");
-		fprintf (header_out, "#undef YY_NO_FLEX_FREE\n");
-		fprintf (header_out, "#undef YY_NO_GET_DEBUG\n");
-		fprintf (header_out, "#undef YY_NO_GET_EXTRA\n");
-		fprintf (header_out, "#undef YY_NO_GET_IN\n");
-		fprintf (header_out, "#undef YY_NO_GET_LENG\n");
-		fprintf (header_out, "#undef YY_NO_GET_LINENO\n");
-		fprintf (header_out, "#undef YY_NO_GET_LLOC\n");
-		fprintf (header_out, "#undef YY_NO_GET_LVAL\n");
-		fprintf (header_out, "#undef YY_NO_GET_OUT\n");
-		fprintf (header_out, "#undef YY_NO_GET_TEXT\n");
-		fprintf (header_out, "#undef YY_NO_INPUT\n");
-		fprintf (header_out, "#undef YY_NO_POP_STATE\n");
-		fprintf (header_out, "#undef YY_NO_PUSH_STATE\n");
-		fprintf (header_out, "#undef YY_NO_SCAN_BUFFER\n");
-		fprintf (header_out, "#undef YY_NO_SCAN_BYTES\n");
-		fprintf (header_out, "#undef YY_NO_SCAN_STRING\n");
-		fprintf (header_out, "#undef YY_NO_SET_DEBUG\n");
-		fprintf (header_out, "#undef YY_NO_SET_EXTRA\n");
-		fprintf (header_out, "#undef YY_NO_SET_IN\n");
-		fprintf (header_out, "#undef YY_NO_SET_LINENO\n");
-		fprintf (header_out, "#undef YY_NO_SET_LLOC\n");
-		fprintf (header_out, "#undef YY_NO_SET_LVAL\n");
-		fprintf (header_out, "#undef YY_NO_SET_OUT\n");
-		fprintf (header_out, "#undef YY_NO_TOP_STATE\n");
-		fprintf (header_out, "#undef YY_NO_UNISTD_H\n");
-		fprintf (header_out, "#undef YY_NO_UNPUT\n");
-		fprintf (header_out, "#undef YY_NULL\n");
-		fprintf (header_out, "#undef YY_NUM_RULES\n");
-		fprintf (header_out, "#undef YY_ONLY_ARG\n");
-		fprintf (header_out, "#undef YY_PROTO\n");
-		fprintf (header_out, "#undef YY_READ_BUF_SIZE\n");
-		fprintf (header_out, "#undef YY_REENTRANT\n");
-		fprintf (header_out, "#undef YY_BISON_BRIDGE\n");
-		fprintf (header_out, "#undef YY_RESTORE_YY_MORE_OFFSET\n");
-		fprintf (header_out, "#undef YY_RULE_SETUP\n");
-		fprintf (header_out, "#undef YY_SC_TO_UI\n");
-		fprintf (header_out, "#undef YY_SKIP_YYWRAP\n");
-		fprintf (header_out, "#undef YY_STACK_USED\n");
-		fprintf (header_out, "#undef YY_START\n");
-		fprintf (header_out, "#undef YY_START_STACK_INCR\n");
-		fprintf (header_out, "#undef YY_STATE_EOF\n");
-		fprintf (header_out, "#undef YY_STDINIT\n");
-		fprintf (header_out, "#undef YY_TEXT_IS_ARRAY\n");
-		fprintf (header_out, "#undef YY_TRAILING_HEAD_MASK\n");
-		fprintf (header_out, "#undef YY_TRAILING_MASK\n");
-		fprintf (header_out, "#undef YY_USER_ACTION\n");
-		fprintf (header_out, "#undef YY_USES_REJECT\n");
-		fprintf (header_out, "#undef YY_USE_CONST\n");
-		fprintf (header_out, "#undef YY_USE_LINENO\n");
-		fprintf (header_out, "#undef YY_USE_PROTOS\n");
-		fprintf (header_out, "#undef unput\n");
-		fprintf (header_out, "#undef yy_create_buffer\n");
-		fprintf (header_out, "#undef yy_delete_buffer\n");
-		fprintf (header_out, "#undef yy_flex_debug\n");
-		fprintf (header_out, "#undef yy_flush_buffer\n");
-		fprintf (header_out, "#undef yy_init_buffer\n");
-		fprintf (header_out, "#undef yy_load_buffer_state\n");
-		fprintf (header_out, "#undef yy_new_buffer\n");
-		fprintf (header_out, "#undef yy_scan_buffer\n");
-		fprintf (header_out, "#undef yy_scan_bytes\n");
-		fprintf (header_out, "#undef yy_scan_string\n");
-		fprintf (header_out, "#undef yy_set_bol\n");
-		fprintf (header_out, "#undef yy_set_interactive\n");
-		fprintf (header_out, "#undef yy_switch_to_buffer\n");
-		fprintf (header_out, "#undef yyconst\n");
-		fprintf (header_out, "#undef yyextra\n");
-		fprintf (header_out, "#undef yyget_debug\n");
-		fprintf (header_out, "#undef yyset_debug\n");
-		fprintf (header_out, "#undef yyget_extra\n");
-		fprintf (header_out, "#undef yyget_in\n");
-		fprintf (header_out, "#undef yyget_leng\n");
-		fprintf (header_out, "#undef yyget_lineno\n");
-		fprintf (header_out, "#undef yyget_lloc\n");
-		fprintf (header_out, "#undef yyget_lval\n");
-		fprintf (header_out, "#undef yyget_out\n");
-		fprintf (header_out, "#undef yyget_text\n");
-		fprintf (header_out, "#undef yyin\n");
-		fprintf (header_out, "#undef yyleng\n");
-		fprintf (header_out, "#undef yyless\n");
-		fprintf (header_out, "#undef yylex\n");
-		fprintf (header_out, "#undef yylex_destroy\n");
-		fprintf (header_out, "#undef yylex_init\n");
-		fprintf (header_out, "#undef yylineno\n");
-		fprintf (header_out, "#undef yylloc\n");
-		fprintf (header_out, "#undef yylval\n");
-		fprintf (header_out, "#undef yymore\n");
-		fprintf (header_out, "#undef yyout\n");
-		fprintf (header_out, "#undef yyrestart\n");
-		fprintf (header_out, "#undef yyset_extra\n");
-		fprintf (header_out, "#undef yyset_in\n");
-		fprintf (header_out, "#undef yyset_lineno\n");
-		fprintf (header_out, "#undef yyset_lloc\n");
-		fprintf (header_out, "#undef yyset_lval\n");
-		fprintf (header_out, "#undef yyset_out\n");
-		fprintf (header_out, "#undef yyterminate\n");
-		fprintf (header_out, "#undef yytext\n");
-		fprintf (header_out, "#undef yytext_ptr\n");
-		fprintf (header_out, "#undef yywrap\n");
-		fprintf (header_out, "#undef yyalloc\n");
-		fprintf (header_out, "#undef yyrealloc\n");
-		fprintf (header_out, "#undef yyfree\n");
+        {
+            const char * undef_list[] = {
+
+                "BEGIN",
+                "ECHO",
+                "EOB_ACT_CONTINUE_SCAN",
+                "EOB_ACT_END_OF_FILE",
+                "EOB_ACT_LAST_MATCH",
+                "FLEX_SCANNER",
+                "FLEX_STD",
+                "REJECT",
+                "YYFARGS0",
+                "YYFARGS1",
+                "YYFARGS2",
+                "YYFARGS3",
+                "YYLMAX",
+                "YYSTATE",
+                "YY_AT_BOL",
+                "YY_BREAK",
+                "YY_BUFFER_EOF_PENDING",
+                "YY_BUFFER_NEW",
+                "YY_BUFFER_NORMAL",
+                "YY_BUF_SIZE",
+                "M4_YY_CALL_LAST_ARG",
+                "M4_YY_CALL_ONLY_ARG",
+                "YY_CURRENT_BUFFER",
+                "YY_DECL",
+                "M4_YY_DECL_LAST_ARG",
+                "M4_YY_DEF_LAST_ARG",
+                "M4_YY_DEF_ONLY_ARG",
+                "YY_DO_BEFORE_ACTION",
+                "YY_END_OF_BUFFER",
+                "YY_END_OF_BUFFER_CHAR",
+                "YY_EXIT_FAILURE",
+                "YY_EXTRA_TYPE",
+                "YY_FATAL_ERROR",
+                "YY_FLEX_DEFINED_ECHO",
+                "YY_FLEX_LEX_COMPAT",
+                "YY_FLEX_MAJOR_VERSION",
+                "YY_FLEX_MINOR_VERSION",
+                "YY_FLEX_SUBMINOR_VERSION",
+                "YY_FLUSH_BUFFER",
+                "YY_G",
+                "YY_INPUT",
+                "YY_INTERACTIVE",
+                "YY_INT_ALIGNED",
+                "YY_LAST_ARG",
+                "YY_LESS_LINENO",
+                "YY_LEX_ARGS",
+                "YY_LEX_DECLARATION",
+                "YY_LEX_PROTO",
+                "YY_MAIN",
+                "YY_MORE_ADJ",
+                "YY_NEED_STRLEN",
+                "YY_NEW_FILE",
+                "YY_NULL",
+                "YY_NUM_RULES",
+                "YY_ONLY_ARG",
+                "YY_PARAMS",
+                "YY_PROTO",
+                "M4_YY_PROTO_LAST_ARG",
+                "M4_YY_PROTO_ONLY_ARG void",
+                "YY_READ_BUF_SIZE",
+                "YY_REENTRANT",
+                "YY_RESTORE_YY_MORE_OFFSET",
+                "YY_RULE_SETUP",
+                "YY_SC_TO_UI",
+                "YY_SKIP_YYWRAP",
+                "YY_START",
+                "YY_START_STACK_INCR",
+                "YY_STATE_EOF",
+                "YY_STDINIT",
+                "YY_TRAILING_HEAD_MASK",
+                "YY_TRAILING_MASK",
+                "YY_USER_ACTION",
+                "YY_USE_CONST",
+                "YY_USE_PROTOS",
+                "unput",
+                "yyTABLES_NAME",
+                "yy_create_buffer",
+                "yy_delete_buffer",
+                "yy_flex_debug",
+                "yy_flush_buffer",
+                "yy_init_buffer",
+                "yy_load_buffer_state",
+                "yy_new_buffer",
+                "yy_scan_buffer",
+                "yy_scan_bytes",
+                "yy_scan_string",
+                "yy_set_bol",
+                "yy_set_interactive",
+                "yy_switch_to_buffer",
+				"yypush_buffer_state",
+				"yypop_buffer_state",
+				"yyensure_buffer_stack",
+                "yyalloc",
+                "yyconst",
+                "yyextra",
+                "yyfree",
+                "yyget_debug",
+                "yyget_extra",
+                "yyget_in",
+                "yyget_leng",
+                "yyget_lineno",
+                "yyget_lloc",
+                "yyget_lval",
+                "yyget_out",
+                "yyget_text",
+                "yyin",
+                "yyleng",
+                "yyless",
+                "yylex",
+                "yylex_destroy",
+                "yylex_init",
+                "yylineno",
+                "yylloc",
+                "yylval",
+                "yymore",
+                "yyout",
+                "yyrealloc",
+                "yyrestart",
+                "yyset_debug",
+                "yyset_extra",
+                "yyset_in",
+                "yyset_lineno",
+                "yyset_lloc",
+                "yyset_lval",
+                "yyset_out",
+                "yytables_destroy",
+                "yytables_fload",
+                "yyterminate",
+                "yytext",
+                "yytext_ptr",
+                "yywrap",
+
+                /* must be null-terminated */
+                NULL};
+
+
+                for (i=0; undef_list[i] != NULL; i++)
+                    fprintf (header_out, "#undef %s\n", undef_list[i]);
+        }
 
 		/* undef any of the auto-generated symbols. */
 		for (i = 0; i < defs_buf.nelts; i++) {
@@ -795,14 +690,14 @@ void flexend (exit_status)
 				headerfilename);
 		fflush (header_out);
 		fclose (header_out);
-	}
+#endif
 
 	if (exit_status != 0 && outfile_created) {
 		if (ferror (stdout))
 			lerrsf (_("error writing output file %s"),
 				outfilename);
 
-		else if (fclose (stdout))
+		else if ((_stdout_closed = 1) && fclose (stdout))
 			lerrsf (_("error closing output file %s"),
 				outfilename);
 
@@ -859,8 +754,10 @@ void flexend (exit_status)
 			putc ('s', stderr);
 		if (reentrant)
 			fputs ("--reentrant", stderr);
-        if (bison_bridge)
+        if (bison_bridge_lval)
             fputs ("--bison-bridge", stderr);
+        if (bison_bridge_lloc)
+            fputs ("--bison-locations", stderr);
 		if (use_stdout)
 			putc ('t', stderr);
 		if (printstats)
@@ -1042,7 +939,7 @@ void flexinit (argc, argv)
 	yymore_really_used = reject_really_used = unspecified;
 	interactive = csize = unspecified;
 	do_yywrap = gen_line_dirs = usemecs = useecs = true;
-	reentrant = bison_bridge = false;
+	reentrant = bison_bridge_lval = bison_bridge_lloc = false;
 	performance_report = 0;
 	did_outfilename = 0;
 	prefix = "yy";
@@ -1051,6 +948,7 @@ void flexinit (argc, argv)
 	tablesext = tablesverify = false;
 	gentables = true;
 	tablesfilename = tablesname = NULL;
+    ansi_func_defs = ansi_func_protos = true;
 
 	sawcmpflag = false;
 
@@ -1064,7 +962,17 @@ void flexinit (argc, argv)
 	buf_init (&userdef_buf, sizeof (char));	/* one long string */
 	buf_init (&defs_buf, sizeof (char *));	/* list of strings */
 	buf_init (&yydmap_buf, sizeof (char));	/* one long string */
+	buf_init (&top_buf, sizeof (char));	    /* one long string */
 
+    {
+        const char * m4defs_init_str[] = {"m4_changequote\n",
+                                          "m4_changequote([[, ]])\n"};
+        buf_init (&m4defs_buf, sizeof (char *));
+        buf_append (&m4defs_buf, &m4defs_init_str, 2);
+    }
+
+    /* initialize regex lib */
+    flex_init_regex();
 
 	/* Enable C++ if program name ends with '+'. */
 	program_name = basename2 (argv[0], 0);
@@ -1189,6 +1097,10 @@ void flexinit (argc, argv)
 			posix_compat = true;
 			break;
 
+        case OPT_PREPROC_LEVEL:
+            preproc_level = strtol(arg,NULL,0);
+            break;
+
 		case OPT_MAIN:
 			buf_strdefine (&userdef_buf, "YY_MAIN", "1");
 			do_yywrap = false;
@@ -1216,7 +1128,11 @@ void flexinit (argc, argv)
 			break;
 
 		case OPT_BISON_BRIDGE:
-			bison_bridge = true;
+			bison_bridge_lval = true;
+			break;
+
+		case OPT_BISON_BRIDGE_LOCATIONS:
+			bison_bridge_lval = bison_bridge_lloc = true;
 			break;
 
 		case OPT_REENTRANT:
@@ -1244,8 +1160,8 @@ void flexinit (argc, argv)
 			break;
 
 		case OPT_NO_UNISTD_H:
-			buf_strdefine (&userdef_buf, "YY_NO_UNISTD_H",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_UNISTD_H", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_UNISTD_H",0);
 			break;
 
 		case OPT_TABLES_FILE:
@@ -1294,13 +1210,11 @@ void flexinit (argc, argv)
 			break;
 
 		case OPT_ALWAYS_INTERACTIVE:
-			buf_strdefine (&userdef_buf,
-				       "YY_ALWAYS_INTERACTIVE", "1");
+			buf_m4_define (&m4defs_buf, "M4_YY_ALWAYS_INTERACTIVE", 0);
 			break;
 
 		case OPT_NEVER_INTERACTIVE:
-			buf_strdefine (&userdef_buf,
-				       "YY_NEVER_INTERACTIVE", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NEVER_INTERACTIVE", 0);
 			break;
 
 		case OPT_ARRAY:
@@ -1361,7 +1275,8 @@ void flexinit (argc, argv)
 			break;
 
 		case OPT_STACK:
-			buf_strdefine (&userdef_buf, "YY_STACK_USED", "1");
+			//buf_strdefine (&userdef_buf, "YY_STACK_USED", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_STACK_USED",0);
 			break;
 
 		case OPT_STDINIT:
@@ -1408,84 +1323,97 @@ void flexinit (argc, argv)
 			reject_really_used = false;
 			break;
 
+        case OPT_NO_ANSI_FUNC_DEFS:
+            ansi_func_defs = false;
+            break;
+
+        case OPT_NO_ANSI_FUNC_PROTOS:
+            ansi_func_protos = false;
+            break;
+
 		case OPT_NO_YY_PUSH_STATE:
-			buf_strdefine (&userdef_buf, "YY_NO_PUSH_STATE",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_PUSH_STATE", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_PUSH_STATE",0);
 			break;
 		case OPT_NO_YY_POP_STATE:
-			buf_strdefine (&userdef_buf, "YY_NO_POP_STATE",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_POP_STATE", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_POP_STATE",0);
 			break;
 		case OPT_NO_YY_TOP_STATE:
-			buf_strdefine (&userdef_buf, "YY_NO_TOP_STATE",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_TOP_STATE", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_TOP_STATE",0);
 			break;
 		case OPT_NO_UNPUT:
-			buf_strdefine (&userdef_buf, "YY_NO_UNPUT", "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_UNPUT", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_UNPUT",0);
 			break;
 		case OPT_NO_YY_SCAN_BUFFER:
-			buf_strdefine (&userdef_buf, "YY_NO_SCAN_BUFFER",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_SCAN_BUFFER", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_SCAN_BUFFER",0);
 			break;
 		case OPT_NO_YY_SCAN_BYTES:
-			buf_strdefine (&userdef_buf, "YY_NO_SCAN_BYTES",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_SCAN_BYTES", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_SCAN_BYTES",0);
 			break;
 		case OPT_NO_YY_SCAN_STRING:
-			buf_strdefine (&userdef_buf, "YY_NO_SCAN_STRING",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_SCAN_STRING", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_SCAN_STRING",0);
 			break;
 		case OPT_NO_YYGET_EXTRA:
-			buf_strdefine (&userdef_buf, "YY_NO_GET_EXTRA",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_GET_EXTRA", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_GET_EXTRA",0);
 			break;
 		case OPT_NO_YYSET_EXTRA:
-			buf_strdefine (&userdef_buf, "YY_NO_SET_EXTRA",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_SET_EXTRA", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_SET_EXTRA",0);
 			break;
 		case OPT_NO_YYGET_LENG:
-			buf_strdefine (&userdef_buf, "YY_NO_GET_LENG",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_GET_LENG", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_GET_LENG",0);
 			break;
 		case OPT_NO_YYGET_TEXT:
-			buf_strdefine (&userdef_buf, "YY_NO_GET_TEXT",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_GET_TEXT", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_GET_TEXT",0);
 			break;
 		case OPT_NO_YYGET_LINENO:
-			buf_strdefine (&userdef_buf, "YY_NO_GET_LINENO",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_GET_LINENO", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_GET_LINENO",0);
 			break;
 		case OPT_NO_YYSET_LINENO:
-			buf_strdefine (&userdef_buf, "YY_NO_SET_LINENO",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_SET_LINENO", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_SET_LINENO",0);
 			break;
 		case OPT_NO_YYGET_IN:
-			buf_strdefine (&userdef_buf, "YY_NO_GET_IN", "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_GET_IN", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_GET_IN",0);
 			break;
 		case OPT_NO_YYSET_IN:
-			buf_strdefine (&userdef_buf, "YY_NO_SET_IN", "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_SET_IN", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_SET_IN",0);
 			break;
 		case OPT_NO_YYGET_OUT:
-			buf_strdefine (&userdef_buf, "YY_NO_GET_OUT", "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_GET_OUT", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_GET_OUT",0);
 			break;
 		case OPT_NO_YYSET_OUT:
-			buf_strdefine (&userdef_buf, "YY_NO_SET_OUT", "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_SET_OUT", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_SET_OUT",0);
 			break;
 		case OPT_NO_YYGET_LVAL:
-			buf_strdefine (&userdef_buf, "YY_NO_GET_LVAL",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_GET_LVAL", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_GET_LVAL",0);
 			break;
 		case OPT_NO_YYSET_LVAL:
-			buf_strdefine (&userdef_buf, "YY_NO_SET_LVAL",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_SET_LVAL", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_SET_LVAL",0);
 			break;
 		case OPT_NO_YYGET_LLOC:
-			buf_strdefine (&userdef_buf, "YY_NO_GET_LLOC",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_GET_LLOC", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_GET_LLOC",0);
 			break;
 		case OPT_NO_YYSET_LLOC:
-			buf_strdefine (&userdef_buf, "YY_NO_SET_LLOC",
-				       "1");
+			//buf_strdefine (&userdef_buf, "YY_NO_SET_LLOC", "1");
+            buf_m4_define( &m4defs_buf, "M4_YY_NO_SET_LLOC",0);
 			break;
 
 		}		/* switch */
@@ -1635,11 +1563,25 @@ void readin ()
 				   ("variable trailing context rules cannot be used with -f or -F"));
 	}
 
-	if (reject)
-		outn ("\n#define YY_USES_REJECT");
+	if (reject){
+        out_m4_define( "M4_YY_USES_REJECT", NULL);
+		//outn ("\n#define YY_USES_REJECT");
+    }
 
 	if (!do_yywrap) {
-		outn ("\n#define yywrap(n) 1");
+              /*
+               * Output different macro defs, since older compilers can't
+               * handle a macro taking a single argument called with none
+               * (which is legal C99)
+               */
+               if(reentrant)
+                {
+                 outn ("\n#define yywrap(n) 1");
+                }
+               else
+                {
+                 outn ("\n#define yywrap() 1"); 
+                }
 		outn ("#define YY_SKIP_YYWRAP");
 	}
 
@@ -1664,15 +1606,16 @@ void readin ()
 		OUT_BEGIN_CODE ();
 		/* In reentrant scanner, stdinit is handled in flex.skl. */
 		if (do_stdinit) {
-			outn ("#ifdef YY_REENTRANT");
-			outn ("#ifdef VMS");
-			outn ("#ifdef __VMS_POSIX");
-			outn ("#define YY_STDINIT");
-			outn ("#endif");
-			outn ("#else");
-			outn ("#define YY_STDINIT");
-			outn ("#endif");
-			outn ("#else /* end YY_REENTRANT */");
+			if (reentrant){
+                outn ("#ifdef VMS");
+                outn ("#ifdef __VMS_POSIX");
+                outn ("#define YY_STDINIT");
+                outn ("#endif");
+                outn ("#else");
+                outn ("#define YY_STDINIT");
+                outn ("#endif");
+            }
+
 			outn ("#ifdef VMS");
 			outn ("#ifndef __VMS_POSIX");
 			outn (yy_nostdinit);
@@ -1682,13 +1625,11 @@ void readin ()
 			outn ("#else");
 			outn (yy_stdinit);
 			outn ("#endif");
-			outn ("#endif");
 		}
 
 		else {
-			outn ("#ifndef YY_REENTRANT");
-			outn (yy_nostdinit);
-			outn ("#endif");
+			if(!reentrant)
+                outn (yy_nostdinit);
 		}
 		OUT_END_CODE ();
 	}
@@ -1898,7 +1839,10 @@ void usage ()
 		  "  -P,  --prefix=STRING     use STRING as prefix instead of \"yy\"\n"
 		  "  -R,  --reentrant         generate a reentrant C scanner\n"
 		  "       --bison-bridge      scanner for bison pure parser.\n"
+		  "       --bison-locations   include yylloc support.\n"
 		  "       --stdinit           initialize yyin/yyout to stdin/stdout\n"
+          "       --noansi-definitions old-style function definitions\n"
+          "       --noansi-prototypes  empty parameter list in prototypes\n"
 		  "       --nounistd          do not include <unistd.h>\n"
 		  "       --noFUNCTION        do not generate a particular FUNCTION\n"
 		  "\n" "Miscellaneous:\n"
